<?php
/**
 * Omnisend Install Class
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

class Omnisend_Install {
	public static function notify_about_plugin_activation() {
		$brand_id = get_option( 'omnisend_account_id', null );
		if ( ! $brand_id ) {
			return;
		}

		$body = array(
			'brandID' => $brand_id,
		);

		Omnisend_Helper::omnisend_api( OMNISEND_ACTIVATION_URL, 'POST', $body );
	}

	public static function notify_about_plugin_update() {
		$brand_id = get_option( 'omnisend_account_id', null );
		if ( ! $brand_id ) {
			return;
		}

		$body = array(
			'brandID' => $brand_id,
		);

		Omnisend_Helper::omnisend_api( OMNISEND_UPDATE_URL, 'POST', $body );
	}

	public static function revoke_omnisend_woo_api_keys() {

		$api_keys = self::get_woo_api_keys();

		if ( sizeof( $api_keys ) <= 0 ) {
			return;
		}

		foreach ( $api_keys as $api_key ) {
			self::remove_woo_api_key( $api_key->key_id );
		}
	}

	public static function delete_omnisend_webhooks() {
		if ( ! is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			return;
		}

		$webhook_data_store = WC_Data_Store::load( 'webhook' );
		$num_webhooks       = $webhook_data_store->get_count_webhooks_by_status();
		$count              = array_sum( $num_webhooks );

		if ( $count <= 0 ) {
			return;
		}

		$webhook_ids = $webhook_data_store->get_webhooks_ids();

		foreach ( $webhook_ids as $webhook_id ) {
			$webhook = wc_get_webhook( $webhook_id );
			if ( ! $webhook ) {
				continue;
			}

			$is_omnisend_delivery_url = false !== strpos( $webhook->get_delivery_url(), 'webhooks-woocommerce.omnisend' );
			$is_omnisend_name         = false !== strpos( $webhook->get_name(), 'omnisend::' );
			if ( $is_omnisend_delivery_url && $is_omnisend_name ) {
				$webhook_data_store->delete( $webhook );
			}
		}
	}

	/**
	 * Retrieves woocommerce api keys for omnisend.
	 *
	 * @return array of {"key_id": integer}
	 */
	public static function get_woo_api_keys() {
		global $wpdb;

		$like = OMNISEND_WC_API_APP_NAME . ' - API %';
		$sql  = $wpdb->prepare( "SELECT `key_id`, `user_id` FROM {$wpdb->prefix}woocommerce_api_keys WHERE `description` LIKE %s", $like );

		// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		return $wpdb->get_results( $sql );
	}

	/**
	 * Remove woocommerce api key.
	 *
	 * @param integer $key_id API Key ID.
	 *
	 * @return boolean
	 */
	public static function remove_woo_api_key( $key_id ) {
		global $wpdb;

		$delete = $wpdb->delete( $wpdb->prefix . 'woocommerce_api_keys', array( 'key_id' => $key_id ), array( '%d' ) );

		return $delete;
	}
}
