<?php
/**
 * Omnisend Settings Page
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

/**
 * Render settings page
 */
function omnisend_show_settings_page() {
	if ( ! Omnisend_Helper::is_woocommerce_plugin_activated() ) {
		display_woocommerce_not_installed_or_disabled();
		return;
	}

	if ( ! Omnisend_Helper::check_wp_wc_compatibility() ) {
		display_unsupported_wordpress_version();
		return;
	}

	if ( get_option( 'omnisend_api_key', null ) === null ) {
		display_connection();
		return;
	}

	if ( get_option( 'omnisend_initial_sync', null ) == null ) {
		Omnisend_Manager_Assistant::init_sync();
		update_option( 'omnisend_initial_sync', date( DATE_ATOM, time() ) );
	}

	omnisend_handle_settings_page_actions();

	display_settings();
}

function omnisend_handle_settings_page_actions() {
	// phpcs:disable WordPress.Security.NonceVerification
	if ( ! isset( $_POST['action'] ) ) {
		return;
	}

	switch ( $_POST['action'] ) {
		case 'omnisend_save_tag':
			if ( isset( $_POST['tag'] ) ) {
				update_option( 'omnisend_contact_tag', sanitize_text_field( wp_unslash( $_POST['tag'] ) ) );
			} else {
				delete_option( 'omnisend_contact_tag' );
			}
			break;
		case 'omnisend_show_checkout_opt_in':
			if ( isset( $_POST['checkout_opt_in'] ) ) {
				update_option( 'omnisend_checkout_opt_in_text', sanitize_text_field( wp_unslash( $_POST['checkout_opt_in'] ) ) );
			} else {
				delete_option( 'omnisend_checkout_opt_in_text' );
			}
			break;
	}
	// phpcs:enable
}

function display_omnisend_logo() {
	?>
	<div class="omnisend-logo">
		<a href="<?php echo esc_url( OMNISEND_APP_URL ); ?>" target="_blank">
			<img src="<?php echo esc_url( plugin_dir_url( __FILE__ ) ) . 'assets/img/logo.svg'; ?>">
		</a>
	</div>
	<?php
}

function display_settings() {
	?>
	<div class="settings-page">
		<?php
		display_omnisend_logo();
		display_permalink_notice();
		display_api_access_notice();
		display_omnisend_connected();
		display_tag_settings();
		display_checkout_opt_in_checkbox_settings();
		display_plugin_version();
		?>
	</div>
	<?php
}

function display_connection() {
	?>
	<div class="connection-container">
		<?php
		$permalinks_correct = true;
		if ( ! Omnisend_Helper::are_permalinks_correct() ) {
			$permalinks_correct = false;
			Omnisend_Plugin_Event::permalinks_notice_shown();
			display_permalink_notice();
		}

		display_connection_error_notice();
		track_oauth_deny();
		display_connection_info( $permalinks_correct );
		?>
	</div>
	<?php
	display_plugin_version();
}

function display_unsupported_wordpress_version() {
	?>
	<div class="settings-page">
		<?php
		display_omnisend_logo();
		?>
		<div class="settings-section">
			<div class="omnisend-notice">
				<div class="omnisend-notice-content-container">
					<img src="<?php echo esc_url( plugin_dir_url( __NAMESPACE__ ) ) . 'omnisend-connect/assets/img/warning_24.svg'; ?>">
					<div class="omnisend-notice-texts-container">
						<p>Your current WordPress version needs an update to support the latest WooCommerce version.</p>
					</div>
				</div>
			</div>
		</div>
		<?php
		display_plugin_version();
		?>
	</div>
	<?php
}

function display_woocommerce_not_installed_or_disabled() {
	?>
	<div class="settings-page">
		<?php
		display_omnisend_logo();
		?>
		<div class="settings-section">
			<div class="omnisend-notice">
				<div class="omnisend-notice-content-container">
					<img src="<?php echo esc_url( plugin_dir_url( __NAMESPACE__ ) ) . 'omnisend-connect/assets/img/warning_24.svg'; ?>">
					<div class="omnisend-notice-texts-container">
						<p>Omnisend goes hand-in-hand with WooCommerce. Make sure you have <a href=" <?php echo esc_url( network_admin_url( 'plugin-install.php?s=woocommerce&tab=search&type=term' ) ); ?>">WooCommerce</a> installed and activated</p>
					</div>
				</div>
			</div>
		</div>
		<?php
		display_plugin_version();
		?>
	</div>
	<?php
}

function display_plugin_version() {
	$omnisend_plugin_version = Omnisend_Helper::omnisend_plugin_version();
	?>
	<div class="plugin-version">
		<p>
			Omnisend Plugin for Woocommerce - v.<?php echo esc_html( $omnisend_plugin_version ); ?>
		<p>
	</div>
	<?php
}

function track_oauth_deny() {
	// phpcs:disable WordPress.Security.NonceVerification
	if ( isset( $_GET['auth-success'] ) ) {
		if ( $_GET['auth-success'] === 'false' ) {
			Omnisend_Plugin_Event::connection_event( Omnisend_Plugin_Event::WOOCOMMERCE_OAUTH_DENY_BUTTON_CLICKED );
		}
	}
	// phpcs:enable
}

?>
