<?php
/**
 * Omnisend WooCommerce Hook Functions
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

$cart_converted     = false;
$picker_product_set = false;

/* PRODUCTS */

add_action( 'woocommerce_new_product', 'omnisend_on_product_change', 100, 1 );
add_action( 'woocommerce_update_product', 'omnisend_on_product_change', 100, 1 );
add_action( 'trash_product', 'omnisend_product_delete' );

/**
 * Product create or update
 *
 * @param int $post_id Product ID.
 */
function omnisend_on_product_change( $post_id ) {
	Omnisend_Logger::hook();
	remove_action( 'woocommerce_update_product', 'omnisend_on_product_change' );
	if ( Omnisend_Helper::is_woocommerce_plugin_activated() ) {
		Omnisend_Manager::push_product_to_omnisend( $post_id );
	}
}

/**
 * Product create or update
 *
 * @param int $post_id Product ID.
 */
function omnisend_product_delete( $post_id ) {
	Omnisend_Logger::hook();
	if ( Omnisend_Helper::is_woocommerce_plugin_activated() ) {
		Omnisend_Manager::delete_product_from_omnisend( $post_id );
	}
}

/* Product page - add Product Picker */
add_action( 'woocommerce_after_single_product', 'omnisend_product_picker', 5 );
function omnisend_product_picker() {
	Omnisend_Logger::hook();
	global $picker_product_set;
	if ( $picker_product_set == false ) {
		$picker_product_set = true;
		Omnisend_Product::product_picker();
	}
}

/* PRODUCT CATEGORIES */

add_action( 'edited_product_cat', 'omnisend_on_category_change', 10, 2 );
add_action( 'create_product_cat', 'omnisend_on_category_change', 10, 2 );
add_action( 'delete_product_cat', 'omnisend_category_delete', 10, 1 );

/**
 * Category create or update
 *
 * @param int $term_id Category ID.
 */
function omnisend_on_category_change( $term_id ) {
	Omnisend_Logger::hook();
	remove_action( 'edited_product_cat', 'omnisend_on_category_change' );
	Omnisend_Manager::push_category_to_omnisend( $term_id );
}

/**
 * Category create or update
 *
 * @param int $post_id Category ID.
 */
function omnisend_category_delete( $post_id ) {
	Omnisend_Logger::hook();
	if ( Omnisend_Helper::is_woocommerce_plugin_activated() ) {
		Omnisend_Manager::delete_category_from_omnisend( $post_id );
	}
}

/* CONTACTS */

add_action( 'profile_update', 'omnisend_on_user_update', 10, 2 );
function omnisend_on_user_update( $user_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::push_contact_to_omnisend( $user_id );
	Omnisend_Contact_Resolver::update_by_user_id( $user_id );
}

add_action( 'user_register', 'omnisend_on_user_register', 10, 1 );
function omnisend_on_user_register( $user_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::push_contact_to_omnisend( $user_id );
	Omnisend_Contact_Resolver::update_by_user_id( $user_id );
}

/* ORDERS */

/* Hook for triggering action when order created */
add_action( 'woocommerce_checkout_update_order_meta', 'omnisend_order_created', 20, 2 );
function omnisend_order_created( $order_id ) {
	Omnisend_Logger::hook();

	global $cart_converted;
	$cart_converted = true;

	// Add cartID to order, if it doesn't exist.
	$cart_id = Omnisend_Server_Session::get( 'omnisend_cartID' );
	if ( $cart_id != '' ) {
		add_post_meta( $order_id, 'omnisend_cartID', $cart_id, true );
	}
	if ( Omnisend_User_Storage::get_attribution_id() ) {
		add_post_meta( $order_id, 'omnisendAttributionID', Omnisend_User_Storage::get_attribution_id(), true );
	}
	Omnisend_Manager::push_order_to_omnisend( $order_id );
	Omnisend_Manager_Assistant::unset_user_cart( true );
}

/* Hook trigered when admin updates order */
add_action( 'woocommerce_process_shop_order_meta', 'omnisend_order_updated', 50, 2 );
function omnisend_order_updated( $order_id ) {
	Omnisend_Logger::hook();
	if ( is_admin() ) {
		Omnisend_Manager::push_order_to_omnisend( $order_id );
	}
}

/*
Fulfillment statuses.
Hook for triggering action when order staus is changed to Processing */
add_action( 'woocommerce_order_status_processing', 'omnisend_order_processing', 10, 1 );
function omnisend_order_processing( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'fulfillment', 'inProgress' );
}

/* Hook for triggering action when order staus is changed to Completed */
add_action( 'woocommerce_order_status_completed', 'omnisend_order_completed', 10, 1 );
function omnisend_order_completed( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'fulfillment', 'fulfilled' );
}

/*
Payment statuses.
Hook for triggering action when order staus is changed to Pending */
add_action( 'woocommerce_order_status_pending', 'omnisend_order_pending', 10, 1 );
function omnisend_order_pending( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'payment', 'awaitingPayment' );
}

/* Hook for triggering action when order staus is changed to Cancelled */
add_action( 'woocommerce_order_status_cancelled', 'omnisend_order_cancelled', 10, 1 );
function omnisend_order_cancelled( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'payment', 'voided' );
}

/* Hook for triggering action when order staus is changed to Refunded */
add_action( 'woocommerce_order_status_refunded', 'omnisend_order_refunded', 10, 1 );
function omnisend_order_refunded( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'payment', 'refunded' );
}

/* Hook for triggering action when order Payment is complete */
add_action( 'woocommerce_payment_complete', 'omnisend_order_payment_completed', 10, 1 );
function omnisend_order_payment_completed( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'payment', 'paid' );
}

/* Hook for triggering action when order Payment failed (order status set to Failed) */
add_action( 'woocommerce_order_status_failed', 'omnisend_order_payment_failed', 10, 1 );
function omnisend_order_payment_failed( $order_id ) {
	Omnisend_Logger::hook();
	Omnisend_Manager::update_order_status( $order_id, 'payment', 'awaitingPayment' );
}

/* CARTS */

add_action( 'woocommerce_after_calculate_totals', 'omnisend_cart_updated', 100, 2 );
function omnisend_cart_updated() {
	Omnisend_Logger::hook();
	global $cart_converted;
	if ( ! $cart_converted ) {
		Omnisend_Manager::push_cart_to_omnisend();
	}
}

add_action( 'woocommerce_cart_item_removed', 'omnisend_cart_delete', 10, 2 );
function omnisend_cart_delete() {
	Omnisend_Logger::hook();
	global $cart_converted;
	if ( ! $cart_converted ) {
		if ( WC()->cart->is_empty() ) {
			Omnisend_Manager::push_cart_to_omnisend();
		}
	}
}

add_action( 'woocommerce_add_to_cart', 'omnisend_track_add_to_cart_event', 25, 0 );
function omnisend_track_add_to_cart_event() {
	Omnisend_Logger::hook();
	Omnisend_Cart_Event::added_product_to_cart();
}

add_action( 'wp_enqueue_scripts', 'omnisend_add_checkout_script' );
function omnisend_add_checkout_script() {
	if ( function_exists( 'is_checkout' ) && is_checkout() ) {
		Omnisend_Logger::hook();
		$file_name = 'omnisend-checkout-script.js';
		$file_path = plugin_dir_url( __FILE__ ) . 'assets/js/' . $file_name . '?' . time();
		wp_register_script( $file_name, $file_path, array(), '1.0.0', true );
		wp_localize_script(
			$file_name,
			'omnisend_checkout_vars',
			array(
				'ajax_url'   => admin_url( 'admin-ajax.php' ),
				'contact_id' => Omnisend_User_Storage::get_contact_id(),
			)
		);
		wp_enqueue_script( $file_name, $file_path, array(), '1.0.0', true );
	}
}

/**
 * Modify order webhook payload
 *
 * Add order status URL from Order via get_view_order_url() method
 *
 * @since 1.11.7
 */
function omnisend_add_order_status_url_to_webhook_payload( $payload, $resource ) {
	if ( $resource !== 'order' ) {
		return $payload;
	}

	$order = wc_get_order( $payload['id'] );
	if ( ! empty( $order ) && $order->get_user() ) {
		$order_url                            = esc_url( $order->get_view_order_url() );
		$payload['omnisend_order_status_url'] = $order_url ? $order_url : null;
	}

	return $payload;
}
add_filter( 'woocommerce_webhook_payload', 'omnisend_add_order_status_url_to_webhook_payload', 10, 4 );

/**
 * Restore cart URL
 */
function omisend_restore_cart_page() {
	// phpcs:disable WordPress.Security.NonceVerification
	if ( isset( $_REQUEST['action'] ) ) {
		Omnisend_Logger::hook();
		if ( $_REQUEST['action'] == 'restoreCart' ) {
			omnisend_restore_cart();
		}
	}
	// phpcs:enable
}

add_action( 'wp', 'omisend_restore_cart_page' );

/**
 * Identify user after login - save cookie
 */
function omnisend_wplogin( $user_login, $user ) {
	Omnisend_Logger::hook();
	Omnisend_Contact_Resolver::update_by_user_id( $user->ID );
}

add_action( 'wp_login', 'omnisend_wplogin', 10, 2 );

/* Add code snippet to the footer, if account ID is setted */
add_action(
	'wp_footer',
	function () {
		global $picker_product_set, $omnisend_plugin_version;

		if ( Omnisend_Helper::is_woocommerce_plugin_activated() && Omnisend_Helper::check_wp_wc_compatibility() ) {

			$omnisend_account_id = get_option( 'omnisend_account_id', null );
			if ( $omnisend_account_id !== null ) {
				?>
				<script type="text/javascript">
					//OMNISEND-SNIPPET-SOURCE-CODE-V1
					window.omnisend = window.omnisend || [];
					omnisend.push(["accountID", "<?php echo esc_attr( get_option( 'omnisend_account_id', null ) ); ?>"]);
					omnisend.push(["track", "$pageViewed"]);
					!function () {
						var e = document.createElement("script");
						e.type = "text/javascript", e.async = !0, e.src = "<?php echo esc_url( OMNISEND_SNIPPET_URL ); ?>";
						var t = document.getElementsByTagName("script")[0];
						t.parentNode.insertBefore(e, t)
					}();
					//platform: woocommerce
					//plugin version: <?php echo esc_js( $omnisend_plugin_version ); ?>
				</script>

				<?php
				if ( is_product() && ! $picker_product_set ) {
					$picker_product_set = true;
					Omnisend_Product::product_picker();
				}
			}
		}
	}
);

/* Add verification tag */
add_action(
	'wp_head',
	function () {
		if ( Omnisend_Helper::is_woocommerce_plugin_activated() && Omnisend_Helper::check_wp_wc_compatibility() ) {

			$omnisend_account_id = get_option( 'omnisend_account_id', null );
			if ( $omnisend_account_id !== null ) {
				?>
				<meta name="omnisend-site-verification" content="<?php echo esc_attr( get_option( 'omnisend_account_id', null ) ); ?>"/>
				<?php
			}
		}
	}
);

function omnisend_checkbox_custom_checkout_field( $checkout ) {
	Omnisend_Logger::hook();
	$option = get_option( 'omnisend_checkout_opt_in_text' );

	woocommerce_form_field(
		'omnisend_newsletter_checkbox',
		array(
			'type'     => 'checkbox',
			'class'    => array( 'omnisend_newsletter_checkbox_field' ),
			'label'    => $option,
			'value'    => true,
			'default'  => 0,
			'required' => false,
		),
		$checkout->get_value( 'omnisend_newsletter_checkbox' )
	);
}

function omnisend_update_contact_status( $order_id ) {
	Omnisend_Logger::hook();
	// phpcs:ignore WordPress.Security.NonceVerification
	if ( isset( $_POST['omnisend_newsletter_checkbox'] ) && sanitize_text_field( wp_unslash( $_POST['omnisend_newsletter_checkbox'] ) ) ) {
		add_post_meta( $order_id, 'marketing_opt_in_consent', 'checkout', true );

		$identifiers = array();

		$billing_email = filter_input( INPUT_POST, 'billing_email', FILTER_SANITIZE_EMAIL );
		if ( $billing_email != '' ) {
			$email_identifier = array(
				'type'     => 'email',
				'id'       => $billing_email,
				'channels' => array(
					'email' => array(
						'status'     => 'subscribed',
						'statusDate' => date( DATE_ATOM, time() ),
					),
				),
			);
			array_push( $identifiers, $email_identifier );
		}

		$billing_phone = filter_input( INPUT_POST, 'billing_phone', FILTER_SANITIZE_NUMBER_INT );
		if ( $billing_phone != '' ) {
			$phone_identifier = array(
				'type'     => 'phone',
				'id'       => $billing_phone,
				'channels' => array(
					'sms' => array(
						'status'     => 'nonSubscribed',
						'statusDate' => date( DATE_ATOM, time() ),
					),
				),
			);
			array_push( $identifiers, $phone_identifier );
		}

		if ( count( $identifiers ) === 0 ) {
			return;
		}

		$prepared_contact = array(
			'identifiers' => $identifiers,
			'tags'        => array(
				'source: checkout',
			),
		);

		$link = OMNISEND_API_URL . 'contacts';
		Omnisend_Helper::omnisend_api( $link, 'POST', $prepared_contact );
	}
}

$omnisend_checkout_opt_in_text = get_option( 'omnisend_checkout_opt_in_text' );

if ( ! empty( $omnisend_checkout_opt_in_text ) ) {
	// Add the checkbox field.
	add_action( 'woocommerce_after_checkout_billing_form', 'omnisend_checkbox_custom_checkout_field' );
	add_action( 'woocommerce_checkout_update_order_meta', 'omnisend_update_contact_status' );
}

function omnisend_plugin_updates() {
	global $omnisend_plugin_version;
	if ( is_admin() && ! empty( get_option( 'omnisend_api_key', null ) ) ) {
		$default_version = '1.0.0';
		$version_db      = get_option( 'omnisend_plugin_version', $default_version );
		$update_info     = false;

		if ( version_compare( $version_db, $omnisend_plugin_version, '<' ) ) {
			$update_info = true;
			if ( $version_db !== $default_version ) {
				Omnisend_Install::notify_about_plugin_update();
				Omnisend_Plugin_Event::plugin_updated();
			}
		}

		if ( $version_db != $omnisend_plugin_version ) {
			update_option( 'omnisend_plugin_version', $omnisend_plugin_version );
			$update_info = true;
		}

		if ( get_option( 'omnisend_wp_version', null ) != get_bloginfo( 'version' ) ) {
			$update_info = true;
		}

		if ( $update_info ) {
			Omnisend_Manager::update_account_info();
		}
	}
}

add_action( 'plugins_loaded', 'omnisend_plugin_updates' );

?>
