<?php
/**
 * Omnisend API Access Notice View
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

function display_api_access_notice() {
	if ( ! Omnisend_Helper::is_woocommerce_api_access_granted() ) {
		Omnisend_Plugin_Event::connection_event( Omnisend_Plugin_Event::UPDATE_PERMISSIONS_OMNISEND_NOTICE_SHOWN );
		display_notice();
	} else {
		display_success();
	}
}

function display_notice() {
	$omnisend_account_id = get_option( 'omnisend_account_id', null );

	if ( $omnisend_account_id !== null ) {
		$full_url = Omnisend_Helper::get_wc_auth_url( $omnisend_account_id );
	}
	?>
	<div>
		<div class="omnisend-notice omnisend-notice-error">
			<div class="omnisend-notice-content-container">
				<img src="<?php echo esc_url( plugin_dir_url( __NAMESPACE__ ) . 'omnisend-connect/assets/img/warning_24.svg' ); ?>">
				<div class="omnisend-notice-texts-container">
					<p class="omnisend-notice-title">Add missing permissions to keep Omnisend working with WooCommerce</p>
					<p class="omnisend-notice-desc">Without your permissions features like, abandoned cart recovery, will not work.</p>
				</div>
			</div>
			<?php
			if ( Omnisend_Helper::are_permalinks_correct() && $omnisend_account_id !== null ) {
				?>
				<button id="omnisend-auth-notice-link" class="omnisend-primary-button danger" data-auth-url="<?php echo esc_url( $full_url ); ?>">Add permissions</button>
				<?php
			} else {
				?>
				<button class="omnisend-primary-button danger disabled">Add permissions</button>
				<?php
			}
			?>
		</div>
	</div>
	<?php
}

function display_success() {
	// phpcs:ignore WordPress.Security.NonceVerification
	if ( empty( $_GET['success'] ) ) {
		return;
	}

	Omnisend_Plugin_Event::connection_event( Omnisend_Plugin_Event::WOOCOMMERCE_OAUTH_SUCCESS );
	?>
	<div class="notice updated is-dismissible">
		<p>You’re all set. Permissions added successfully.</p>
	</div>
	<?php
}
