<?php
/**
 * Omnisend Connection View
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

function display_connection_info( $permalinks_correct ) {
	$install_url = generate_install_url();
	?>
<div class="connection-info-container">
	<div class="connection-content-left">
		<p class="title-text">Welcome to Omnisend</p>
		<p class="sub-title-text">Send personalized and automated emails, collect subscribers, and turn 1st-time visitors into lifelong customers.</p>
		<div class="connection-btns">
			<a target="_blank" href="<?php echo esc_url( generate_login_url( $install_url ) ); ?>" class="
												<?php
												if ( ! $permalinks_correct ) {
													echo 'disabled';}
												?>
			">
				<button id="connect-your-account" class="connect-account-btn">Connect your account</button>
			</a>
			<a target="_blank" href="<?php echo esc_url( generate_registration_url( $install_url ) ); ?>" class="
												<?php
												if ( ! $permalinks_correct ) {
													echo 'disabled';}
												?>
			">
				<button id="create-new-account" class="create-account-btn">Create new account</button>
			</a>
		</div>
		<a target="_blank" href="https://support.omnisend.com/en/articles/1636174-omnisend-for-woocommerce-wordpress" class="omnisend-kb-link">
			Check out article how to integrate with Omnisend 
			<img src="<?php echo esc_url( plugin_dir_url( __NAMESPACE__ ) ) . 'omnisend-connect/assets/img/kb-icon.svg'; ?>">
		</a>
	</div>
	<div class="connection-content-right">
		<img class="connection-image" src="<?php echo esc_url( plugin_dir_url( __NAMESPACE__ ) ) . 'omnisend-connect/assets/img/connection.png'; ?>">
	</div>
</div>
<script type="text/javascript">
	(function(){
		const connectYourAccountButton = document.getElementById('connect-your-account')
		const createNewAccountButton = document.getElementById('create-new-account')
		let interval

		function waitUntilAPIKeyAndRefresh() {
			if (interval) {
				return
			}

			interval = setInterval(async () => {
				fetch("<?php echo esc_url( home_url( '/wp-json/omnisend-api/v1/connected' ) ); ?>").then(r => r.json()).then(connected => {
					if (!connected) {
						return
					}
					clearInterval(interval)
					location.reload()
					connectYourAccountButton.removeEventListener('click', waitUntilAPIKeyAndRefresh)
					createNewAccountButton.removeEventListener('click', waitUntilAPIKeyAndRefresh)
				})
			}, 1000)
		}

		connectYourAccountButton.addEventListener('click', waitUntilAPIKeyAndRefresh)
		createNewAccountButton.addEventListener('click', waitUntilAPIKeyAndRefresh)
	})()
</script>
	<?php
}

function generate_install_url() {
	$token = get_option( 'omnisend_connect_token', '' );

	if ( $token === '' ) {
		$token = hash( 'sha256', time() );
		update_option( 'omnisend_connect_token', $token );
	}

	$install_url_params = array(
		'token'             => $token,
		'storeUrl'          => home_url(),
		'woocommerceUserId' => get_current_user_id(),
	);

	return OMNISEND_PLUGIN_INSTALL_URL . '?' . http_build_query( $install_url_params );
}

function generate_login_url( $install_url ) {
	$login_url_params = array(
		'url' => '/' . $install_url,
	);

	return OMNISEND_LOGIN_URL . '?' . http_build_query( $login_url_params );
}

function generate_registration_url( $install_url ) {
	$registration_url_params = array(
		'utm_source'                => 'woocommerce_plugin',
		'utm_medium'                => 'referral',
		'registration_redirect_url' => $install_url,
	);

	return OMNISEND_REGISTRATION_URL . '?' . http_build_query( $registration_url_params );
}
?>
