<?php
/**
 * Omnisend Sync View
 *
 * @package OmnisendPlugin
 */

defined( 'ABSPATH' ) || exit;

function omnisend_show_sync() {
	Omnisend_Plugin_Event::sync_settings_shown();

	$all_sync_stats = ( new Omnisend_Sync_Stats_Repository() )->get_all_stats();
	$show_error     = has_sync_stats_error( $all_sync_stats );
	$show_skipped   = has_sync_stats_skipped( $all_sync_stats );

	omnisend_handle_sync_page_actions();
	?>

	<div class="settings-page">
		<div class="omnisend-logo">
			<a href="<?php echo esc_url( OMNISEND_APP_URL ); ?>" target="_blank">
				<img alt="Omnisend logo" src="<?php echo esc_url( plugin_dir_url( __FILE__ ) ) . '../../assets/img/logo.svg'; ?>">
			</a>
		</div>
		<div class="settings-section">
			<h3 class="title">Sync</h3>
			<p class="title-desc">Your store data, like contacts and orders, are automatically synced with Omnisend. The
				chart below displays the current sync status.</p>
			<div class="sync-stats">
				<table class="wp-list-table widefat fixed striped posts">
					<thead>
					<tr>
						<td>Data type</td>
						<td class="fixed_date">Successfully synced</td>
						<td>Total</td>
						<td>Pending</td>
						<?php
						if ( $show_error ) {
							echo '<td>Error</td>';
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td>Skipped</td>';
						}
						?>
					</tr>
					</thead>
					<tr>
						<td>Contacts</td>
						<td id="contact-sync-success-count"><?php echo esc_html( $all_sync_stats->contacts->synced ); ?></td>
						<td id="contact-sync-total-count">
							<?php
							if ( $all_sync_stats->contacts->unique && $all_sync_stats->contacts->unique != $all_sync_stats->contacts->total ) {
								echo esc_html( $all_sync_stats->contacts->total ) . ' (Unique - ' . esc_html( $all_sync_stats->contacts->unique ) . ')';
							} else {
								echo esc_html( $all_sync_stats->contacts->total );
							}
							?>
						</td>
						<?php
						if ( $all_sync_stats->contacts->not_synced > 0 ) {
							echo '<td id="contact-sync-pending-count" class="omnisend-warn">' . esc_html( $all_sync_stats->contacts->not_synced ) . '</td>';
						} else {
							echo '<td id="contact-sync-pending-count">' . esc_html( $all_sync_stats->contacts->not_synced ) . '</td>';
						}
						?>
						<?php
						if ( $show_error ) {
							if ( $all_sync_stats->contacts->error > 0 ) {
								echo '<td id="contact-sync-error-count" class="omnisend-warn">' . esc_html( $all_sync_stats->contacts->error ) . '</td>';
							} else {
								echo '<td id="contact-sync-error-count">' . esc_html( $all_sync_stats->contacts->error ) . '</td>';
							}
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td id="contact-sync-skipped-count">' . esc_html( $all_sync_stats->contacts->skipped ) . '</td>';
						}
						?>
					</tr>
					<tr>
						<td>Orders</td>
						<td id="order-sync-success-count"><?php echo esc_html( $all_sync_stats->orders->synced ); ?></td>
						<td id="order-sync-total-count"><?php echo esc_html( $all_sync_stats->orders->total ); ?></td>
						<?php
						if ( $all_sync_stats->orders->not_synced > 0 ) {
							echo '<td id="order-sync-pending-count" class="omnisend-warn">' . esc_html( $all_sync_stats->orders->not_synced ) . '</td>';
						} else {
							echo '<td id="order-sync-pending-count">' . esc_html( $all_sync_stats->orders->not_synced ) . '</td>';
						}
						?>
						<?php
						if ( $show_error ) {
							if ( $all_sync_stats->orders->error > 0 ) {
								echo '<td id="order-sync-error-count" class="omnisend-warn">' . esc_html( $all_sync_stats->orders->error ) . '</td>';
							} else {
								echo '<td id="order-sync-error-count">' . esc_html( $all_sync_stats->orders->error ) . '</td>';
							}
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td id="order-sync-skipped-count">' . esc_html( $all_sync_stats->orders->skipped ) . '</td>';
						}
						?>
					</tr>
					<tr>
						<td>Products</td>
						<td id="product-sync-success-count"><?php echo esc_html( $all_sync_stats->products->synced ); ?></td>
						<td id="product-sync-total-count"><?php echo esc_html( $all_sync_stats->products->total ); ?></td>
						<?php
						if ( $all_sync_stats->products->not_synced > 0 ) {
							echo '<td id="product-sync-pending-count" class="omnisend-warn">' . esc_html( $all_sync_stats->products->not_synced ) . '</td>';
						} else {
							echo '<td id="product-sync-pending-count">' . esc_html( $all_sync_stats->products->not_synced ) . '</td>';
						}
						?>
						<?php
						if ( $show_error ) {
							if ( $all_sync_stats->products->error > 0 ) {
								echo '<td id="product-sync-error-count" class="omnisend-warn">' . esc_html( $all_sync_stats->products->error ) . '</td>';
							} else {
								echo '<td id="product-sync-error-count">' . esc_html( $all_sync_stats->products->error ) . '</td>';
							}
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td id="product-sync-skipped-count">' . esc_html( $all_sync_stats->products->skipped ) . '</td>';
						}
						?>
					</tr>
					<tr>
						<td>Categories</td>
						<td id="category-sync-success-count"><?php echo esc_html( $all_sync_stats->categories->synced ); ?></td>
						<td id="category-sync-total-count"><?php echo esc_html( $all_sync_stats->categories->total ); ?></td>
						<?php
						if ( $all_sync_stats->categories->not_synced > 0 ) {
							echo '<td id="category-sync-pending-count" class="omnisend-warn">' . esc_html( $all_sync_stats->categories->not_synced ) . '</td>';
						} else {
							echo '<td id="category-sync-pending-count">' . esc_html( $all_sync_stats->categories->not_synced ) . '</td>';
						}
						?>
						<?php
						if ( $show_error ) {
							if ( $all_sync_stats->categories->error > 0 ) {
								echo '<td id="category-sync-error-count" class="omnisend-warn">' . esc_html( $all_sync_stats->categories->error ) . '</td>';
							} else {
								echo '<td id="category-sync-error-count">' . esc_html( $all_sync_stats->categories->error ) . '</td>';
							}
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td id="category-sync-skipped-count">' . esc_html( $all_sync_stats->categories->skipped ) . '</td>';
						}
						?>
					</tr>
					<tr>
						<td id="cart-sync-success-count">Carts</td>
						<td><?php echo esc_html( $all_sync_stats->carts ); ?></td>
						<td>-</td>
						<td>-</td>
						<?php
						if ( $show_error ) {
							echo '<td>-</td>';
						}
						?>
						<?php
						if ( $show_skipped ) {
							echo '<td>-</td>';
						}
						?>
					</tr>
				</table>
			</div>
			<?php
			display_sync_actions( $all_sync_stats );
			display_resync_all_contacts();
			?>
		</div>
	</div>
	<?php
}

function omnisend_handle_sync_page_actions() {
	// phpcs:disable WordPress.Security.NonceVerification
	if ( ! isset( $_POST['action'] ) ) {
		return;
	}

	switch ( $_POST['action'] ) {
		case 'omnisend_init_resync':
			Omnisend_Sync_Manager::start_resync_all_with_error_or_skipped();
			break;
		case 'omnisend_resync_all_contacts':
			Omnisend_Sync_Manager::start_resync_contacts();
			break;
	}
	// phpcs:enable
}

function has_sync_stats_error( $sync_stats ) {
	if ( $sync_stats->contacts->error ) {
		return true;
	}
	if ( $sync_stats->orders->error ) {
		return true;
	}
	if ( $sync_stats->products->error ) {
		return true;
	}
	if ( $sync_stats->categories->error ) {
		return true;
	}

	return false;
}

function has_sync_stats_skipped( $sync_stats ) {
	if ( $sync_stats->contacts->skipped ) {
		return true;
	}
	if ( $sync_stats->orders->skipped ) {
		return true;
	}
	if ( $sync_stats->products->skipped ) {
		return true;
	}
	if ( $sync_stats->categories->skipped ) {
		return true;
	}

	return false;
}

function has_sync_stats_not_synced( $sync_stats ) {
	if ( $sync_stats->contacts->not_synced ) {
		return true;
	}
	if ( $sync_stats->orders->not_synced ) {
		return true;
	}
	if ( $sync_stats->products->not_synced ) {
		return true;
	}
	if ( $sync_stats->categories->not_synced ) {
		return true;
	}

	return false;
}

function display_sync_actions( $all_sync_stats ) {
	if ( ! has_sync_stats_error( $all_sync_stats ) && ! has_sync_stats_skipped( $all_sync_stats ) && ! has_sync_stats_not_synced( $all_sync_stats ) ) {
		return;
	}
	?>
	<div class="sync-actions">
		<?php
		if ( Omnisend_Sync_Manager::are_data_syncing() ) {
			?>
			<div class="sync-loader">
				<div class="sync-spinner"></div>
				<span>Syncing...</span>
			</div>
			<?php
		} else {
			?>
			<p>Resync store data from Pending, Error or Skipped columns.</p>
			<div>
				<form method="post">
					<input type="hidden" name="action" value="omnisend_init_resync"/>
					<input type="submit" value="Resync" class="button button-primary"/>
				</form>
			</div>
			<?php
		}
		?>
	</div>
	<?php
}

function display_resync_all_contacts() {
	if ( Omnisend_Sync_Manager::are_contacts_syncing() ) {
		return;
	}
	?>
	<div class="resync-contacts">
		<h3 class="sub-title">Resync all contacts</h3>
		<p class="title-desc">Resync all of your contacts with Omnisend.
			The resync time depends on how many contacts you have.
		</p>
		<form method="post">
			<input type="hidden" name="action" value="omnisend_resync_all_contacts">
			<input type="submit" value="Resync all contacts" class="button button-danger">
		</form>
	</div>
	<?php
}
